import React, { useCallback, useEffect, useRef, useState } from 'react';

import {
  Row,
  TableOptions,
  useColumnOrder,
  useFilters,
  useFlexLayout,
  useGlobalFilter,
  usePagination,
  useResizeColumns,
  useRowSelect,
  useSortBy,
  useTable,
} from 'react-table';
import { useSticky } from './plugins/useSticky';
import TableContainer from '@material-ui/core/TableContainer';
import TableToolbar from 'Toolbar/TableToolbar';
import styles from './EnhancedTable.module.scss';
import clsx from 'clsx';
import { SelectableGroup } from 'react-selectable-fast/lib';
import ReactTablePagination from 'Pagination/ReactTablePagination';
import { ContextMenu, ContextMenuTrigger } from 'react-contextmenu';
import ConditionalWrap from 'utils/ConditionalWrap';
import { Toolbar } from '@material-ui/core';
import config from 'config';
import { useDispatch, useSelector } from 'react-redux';
import { selectMultiRows } from 'store/actions';
import { tableSelectableSelector } from 'store/selectors';
import debounce from 'lodash/debounce';
import selectionHook from 'plugins/selectionHook';
import EditableCellTextField from 'EditableCell/EditableCellTextField';
import RowSelectable from './components/RowSelectable';
import TableHeader from 'components/TableHeader';
import { SELECTION_COLUMN_ID } from 'constants/constants';
import { useLocalStorageTable } from 'plugins/useLocalStorageTable';
import { getStorageInitialState } from 'helpers/initialState';
import { useHiddenColumn } from 'plugins/useHiddenColumn';
import useMainResize from 'utils/customHook/useMainResize';

// Set our editable cell renderer as the default Cell renderer
const defaultColumn = {
  Cell: EditableCellTextField,
};

type Props<T extends { id?: string }> = TableOptions<T> & {
  toolbarItems?: React.ReactElement;
  useSelection?: boolean;
  readonly localStorageKey: string;
  rightClickMenu?: React.ReactElement;
  useBlockingOverlay?: boolean;
  overlayConfig?: {
    displayFullscreen?: boolean;
  };
  alignItemsCenter?: boolean;
  hasGlobalSearch?: boolean;
};

const LocalTable = <T extends Record<string, any>>(props: Props<T>) => {
  const {
    columns,
    data,
    rightClickMenu,
    toolbarItems,
    localStorageKey,
    useSelection = true,
    useBlockingOverlay = true,
    hasGlobalSearch = true,
    overlayConfig,
    alignItemsCenter,
  } = props;

  const initState = getStorageInitialState<T>(localStorageKey);

  const instance = useTable(
    {
      columns,
      data,
      defaultColumn,
      initialState: {
        pageIndex: 0,
        pageSize: config.perPage,
        ...initState,
      },
      pageCount: 100,
    },
    useFlexLayout,
    useResizeColumns,
    useFilters,
    useColumnOrder,
    useGlobalFilter,
    useSortBy,
    usePagination,
    useRowSelect,
    useSticky,
    useLocalStorageTable,
    useHiddenColumn,
    ...(useSelection ? [selectionHook] : [])
  );

  const {
    getTableProps,
    prepareRow,
    page,
    gotoPage,
    setPageSize,
    selectedFlatRows,
    toggleAllRowsSelected,
    rows,
    state: { pageIndex, pageSize },
  } = instance;

  const { width: mainWidth } = useMainResize();
  const stableColumns = [SELECTION_COLUMN_ID];

  const dispatch = useDispatch();

  // eslint-disable-next-line react-hooks/exhaustive-deps
  const delayedQuery = useCallback(
    debounce((rows: any) => {
      dispatch(selectMultiRows(rows));
    }, 500),
    [dispatch]
  );

  useEffect(() => {
    const rows = selectedFlatRows.map((item) => item.original);
    delayedQuery(rows);
  }, [delayedQuery, selectedFlatRows]);

  const tableRef = useRef<HTMLDivElement>(null);
  useEffect(() => {
    if (tableRef.current) {
      tableRef.current.scrollIntoView({ behavior: 'smooth' });
    }
  }, [pageIndex, pageSize]);
  const Pagination = (
    <ReactTablePagination
      gotoPage={gotoPage}
      pageIndex={pageIndex}
      setPageSize={setPageSize}
      pageSize={pageSize}
      pageTotal={data.length}
    />
  );

  const isSelectable = useSelector(tableSelectableSelector);
  const [previousRow, setPreviousRow] = useState<Row | null>(null);

  const handleSelectableGroupFinish = (
    selectedRows: Array<{ props: { row: Row } }>
  ) => {
    if (selectedRows && selectedRows.length > 0) {
      selectedRows.forEach((row) => {
        if (row && row.props) {
          row.props.row.toggleRowSelected();
        }
      });
    }
  };

  // Render the UI for your table
  return (
    <TableContainer
      className={clsx(
        styles.tableContainer,
        useBlockingOverlay &&
          !overlayConfig?.displayFullscreen &&
          styles.positionRelative
      )}
      style={{ width: instance.totalVisibleColumnsWidth }}
    >
      <TableToolbar
        width={mainWidth}
        instance={instance}
        stableColumns={stableColumns}
        toolbarItems={toolbarItems}
        pagination={Pagination}
        hasGlobalSearch={hasGlobalSearch}
        isLocal={true}
      />
      <div className={styles.actionScroll}>
        <SelectableGroup
          className={styles.fixWidthScroll}
          disabled={isSelectable}
          deselectOnEsc={true}
          selectOnClick={false}
          enableDeselect={true}
          allowClickWithoutSelected={false}
          tolerance={5}
          resetOnStart={true}
          onSelectionFinish={handleSelectableGroupFinish}
          ignoreList={['.th']}
        >
          {rightClickMenu && (
            <ContextMenu
              id="same_unique_identifier"
              className={styles.contextMenu}
            >
              {rightClickMenu}
            </ContextMenu>
          )}
          <div className={clsx(styles.tableBodyScroll, styles.noHighLight)}>
            <ConditionalWrap
              condition={!!rightClickMenu}
              wrap={(children) => (
                <ContextMenuTrigger
                  id={'same_unique_identifier'}
                  holdToDisplay={-1}
                >
                  {children}
                </ContextMenuTrigger>
              )}
            >
              <div
                ref={tableRef}
                {...getTableProps()}
                className={clsx(styles.table, styles.sticky)}
              >
                <TableHeader
                  instance={instance}
                  stableColumns={stableColumns}
                />
                <div
                  {...instance.getTableBodyProps()}
                  style={{ width: instance.totalVisibleColumnsWidth }}
                  className={clsx({
                    [styles.alignItemsCenter]: data.length && alignItemsCenter,
                  })}
                >
                  {data.length > 0 ? (
                    page.map((row) => {
                      prepareRow(row);
                      return (
                        <RowSelectable
                          {...row.getRowProps()}
                          row={row}
                          previousRow={previousRow}
                          setPreviousRow={setPreviousRow}
                          toggleAllRowsSelected={toggleAllRowsSelected}
                          rows={rows}
                        />
                      );
                    })
                  ) : (
                    <div className="p-14">No record found</div>
                  )}
                </div>
              </div>
            </ConditionalWrap>
          </div>
        </SelectableGroup>
      </div>
      <Toolbar className={clsx(styles.toolbar)} style={{ width: mainWidth }}>
        {selectedFlatRows.length > 0 && (
          <span>{selectedFlatRows.length} Selected Rows</span>
        )}
        {Pagination}
      </Toolbar>
    </TableContainer>
  );
};

export default LocalTable;
